/*----------------------------------------------------------------------------*-

  ttrd2-08e-t0405a-v001a_processor.c (Release 2017-08-25a)

  Configures Processor in one of the following Modes / States after a reset:

    FAIL_SAFE_S
    NORMAL_M

  Implements rudimentary Fail-Safe State for traffic-light example. 

-*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*-

  This code is copyright (c) 2014-2016 SafeTTy Systems Ltd.

  This code forms part of a Time-Triggered Reference Design (TTRD) that is 
  documented in the following book: 

   Pont, M.J. (2016) 
   "The Engineering of Reliable Embedded Systems (Second Edition)", 
   Published by SafeTTy Systems Ltd. ISBN: 978-0-9930355-3-1.

  Both the TTRDs and the above book ("ERES2") describe patented 
  technology and are subject to copyright and other restrictions.

  This code may be used without charge: [i] by universities and colleges on 
  courses for which a degree up to and including 'MSc' level (or equivalent) 
  is awarded; [ii] for non-commercial projects carried out by individuals 
  and hobbyists.

  Any and all other use of this code and / or the patented technology 
  described in ERES2 requires purchase of a ReliabiliTTy Technology Licence:
  http://www.safetty.net/technology/reliabilitty-technology-licences

  Please contact SafeTTy Systems Ltd if you require clarification of these 
  licensing arrangements: http://www.safetty.net/contact

  CorrelaTTor, DecomposiTTor, DuplicaTTor, MoniTTor, PredicTTor, ReliabiliTTy,  
  SafeTTy, SafeTTy Systems, TriplicaTTor and WarranTTor are registered 
  trademarks or trademarks of SafeTTy Systems Ltd in the UK & other countries.

-*----------------------------------------------------------------------------*/

// Processor Header
#include "..\main\main.h"

// Tasks
#include "..\tasks\ttrd2-08e-t0405a-v001a_tlights_task.h"
#include "..\tasks\ttrd2-02a-t0405a-v001a_iwdt_task.h"

// ------ Private variable definitions -----------------------------

// The current Processor Mode / State
static eProc_MoSt Processor_MoSt_g;

// ------ Private function declarations ----------------------------

void PROCESSOR_Identify_Reqd_MoSt(void);
void PROCESSOR_Configure_Reqd_MoSt(void);

/*----------------------------------------------------------------------------*-

  PROCESSOR_Init()

  Wrapper for system startup functions.
  
  PARAMETERS:
     None.

  LONG-TERM DATA:
     None.

  MCU HARDWARE:
     None.

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  RETURN VALUE:
     None.

-*----------------------------------------------------------------------------*/
void PROCESSOR_Init(void)
   {
   PROCESSOR_Identify_Reqd_MoSt();
   PROCESSOR_Configure_Reqd_MoSt();
   }

/*----------------------------------------------------------------------------*-

  PROCESSOR_Identify_Reqd_MoSt()

  Identify the cause of the system reset.
  Set the system mode / state accordingly.

  PARAMETERS:
     None.

  LONG-TERM DATA:
     Processor_MoSt_g (W)

  MCU HARDWARE:
     Registers indicating cause of reset (R).

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     Check for reset caused by iWDT => 'fail safe'

  RETURN VALUE:
     None.
   
-*----------------------------------------------------------------------------*/
void PROCESSOR_Identify_Reqd_MoSt(void)
   {
   // Check cause of reset
   if (RCC_GetFlagStatus(RCC_FLAG_IWDGRST) == SET)
      {
      // Reset was caused by WDT => "Fail Silent" state
      Processor_MoSt_g = FAIL_SAFE_S;
      }
   else
      {
      // Here we treat all other forms of reset in the same way
      // => "Normal" mode
      Processor_MoSt_g = NORMAL_M;
      }
      
   // Clear cause-of-reset flags
   RCC_ClearFlag();      
   }

/*----------------------------------------------------------------------------*-

  PROCESSOR_Configure_Reqd_MoSt()

  Configure the system in the required mode.  

  PARAMETERS:
     None.

  LONG-TERM DATA:
     Processor_MoSt_g (R)

  MCU HARDWARE:
     None.

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  RETURN VALUE:
     None.
   
-*----------------------------------------------------------------------------*/
void PROCESSOR_Configure_Reqd_MoSt(void)
   {
   switch (Processor_MoSt_g)
      {
      // Default to "Fail Safe" state 
      default:          
      case FAIL_SAFE_S:
         {
         // Reset caused by iWDT
         // Trigger "fail safe" behaviour
         PROCESSOR_Perform_Safe_Shutdown();

         break;
         }

      // NORMAL mode
      case NORMAL_M:
         {
         // Set up the scheduler for 1 ms ticks (tick interval in *ms*)
         SCH_Init_Milliseconds(1);

         // Set up WDT 
         // Timeout is parameter * 100 s: 25 => ~2.5 ms
         // NOTE: WDT driven by RC oscillator - timing varies with temperature 
         WATCHDOG_Init(25);

         // Prepare for heartbeat task
         TLIGHTS_Init(RED);

         // Add tasks to schedule.
         // Parameters are:
         // A. Task name
         // B. Initial delay / offset (in Ticks)
         // C. Task period (in Ticks): Must be > 0
         //           A                    B  C
         SCH_Add_Task(WATCHDOG_Update,     0, 1);    // Feed watchdog
         SCH_Add_Task(TLIGHTS_Update,      0, 1000); // 

         // Feed the watchdog
         WATCHDOG_Update();

         break;
         }
      }
   }

/*----------------------------------------------------------------------------*-

  PROCESSOR_Perform_Safe_Shutdown()

  Attempt to move the system into a safe ('silent') state.

  In this case, we try to flash all of the 'Traffic lights' in a manners
  that would not be expected by a driver in normal operation.

  PARAMETERS:
     None.

  LONG-TERM DATA:
     None.

  MCU HARDWARE:
     Heartbeat LED (GPIO pin).

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  RETURN VALUE:
     None.
   
-*----------------------------------------------------------------------------*/
void PROCESSOR_Perform_Safe_Shutdown(void)
   {
   static uint32_t State_s = 0;
   uint32_t Delay1, Delay2;

   // Here we simply "fail silent" with rudimentary fault reporting.
   // OTHER BEHAVIOUR IS LIKELY TO BE REQUIRED IN YOUR DESIGN

   // *************************************
   // NOTE: This function should NOT return
   // *************************************

   // Set up Heartbeat LED pin
   TLIGHTS_Init(RED);

   while(1)
      {
      // Flash lights to indicate fault
      for (Delay1 = 0; Delay1 < 20000000; Delay1++) 
         {
         Delay2 *= 3;
         }

      if (State_s == 0)
         {
         GPIO_SetBits(RED_LED_PORT, RED_LED_PIN);
         GPIO_SetBits(AMBER_LED_PORT, AMBER_LED_PIN);
         GPIO_SetBits(GREEN_LED_PORT, GREEN_LED_PIN);
         State_s = 1;
         }
      else
         {
         GPIO_ResetBits(RED_LED_PORT, RED_LED_PIN);
         GPIO_ResetBits(AMBER_LED_PORT, AMBER_LED_PIN);
         GPIO_ResetBits(GREEN_LED_PORT, GREEN_LED_PIN);
         State_s = 0;
         }
      }
   }

/*----------------------------------------------------------------------------*-
  ------------------------------ END OF FILE ---------------------------------
-*----------------------------------------------------------------------------*/
