/*----------------------------------------------------------------------------*-

  ttrd2-08e-t0405a-v001a_tlights_task.c (Release 2017-08-25a)

  ----------------------------------------------------------------------------
   
  Example of a Processor Task with multiple Sub-Modes.
  
  Targets DuplicaTTor Evaluation Board (STM32F405, v002).

  Code can be loaded onto either MCU.

  *** PLEASE NOTE - THIS IS A VERY SIMPLE DEMO ***
  *** The goal here is to illustrate the core software architecture ONLY ***
  *** Run-time monitoring (e.g. of bulb outputs) is 'simulated' ***
  *** Fault-injection mechanism is rudimentary ***
  *** At code level, no DuVs, no reg config checks, etc ***

  See ERES2 (Chapter 8) for further information.

  [Based on an example from book 'Embedded C' by Michael J. Pont.]

-*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*-

  This code is copyright (c) 2014-2016 SafeTTy Systems Ltd.

  This code forms part of a Time-Triggered Reference Design (TTRD) that is 
  documented in the following book: 

   Pont, M.J. (2016) 
   "The Engineering of Reliable Embedded Systems (Second Edition)", 
   Published by SafeTTy Systems Ltd. ISBN: 978-0-9930355-3-1.

  Both the TTRDs and the above book ("ERES2") describe patented 
  technology and are subject to copyright and other restrictions.

  This code may be used without charge: [i] by universities and colleges on 
  courses for which a degree up to and including 'MSc' level (or equivalent) 
  is awarded; [ii] for non-commercial projects carried out by individuals 
  and hobbyists.

  Any and all other use of this code and / or the patented technology 
  described in ERES2 requires purchase of a ReliabiliTTy Technology Licence:
  http://www.safetty.net/technology/reliabilitty-technology-licences

  Please contact SafeTTy Systems Ltd if you require clarification of these 
  licensing arrangements: http://www.safetty.net/contact

  CorrelaTTor, DecomposiTTor, DuplicaTTor, MoniTTor, PredicTTor, ReliabiliTTy,  
  SafeTTy, SafeTTy Systems, TriplicaTTor and WarranTTor are registered 
  trademarks or trademarks of SafeTTy Systems Ltd in the UK & other countries.

-*----------------------------------------------------------------------------*/

// Module header
#include "ttrd2-08e-t0405a-v001a_tlights_task.h"

// ------ Private constants --------------------------------------------------

// Times (in seconds) in each of the (four) possible light Sub-Modes
#define RED_DURATION            10
#define RED_AND_AMBER_DURATION  5
#define GREEN_DURATION          10
#define AMBER_DURATION          5

// ------ Private variable definitions ---------------------------------------

// The current Sub-Mode
eTLight_Sub_Mode Tlight_sub_mode_g;

/*----------------------------------------------------------------------------*-

  TLIGHTS_Init()

  Prepare for TLIGHTS_Update() function - see below.
  
  PARAMETERS:
     None.

  LONG-TERM DATA:
     None.

  MCU HARDWARE:
     3 GPIO pins.

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  RETURN VALUE:
     None.

-*----------------------------------------------------------------------------*/
void TLIGHTS_Init(const eTLight_Sub_Mode START_STATE)
   {
   GPIO_InitTypeDef GPIO_InitStruct;
   
   // ---
   // Red LED

   // Enable peripheral clock 
   RCC_AHB1PeriphClockCmd(RED_LED_CLOCK, ENABLE);
     
   // Configure port pin for the LED
   GPIO_InitStruct.GPIO_Mode  = GPIO_Mode_OUT;
   GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
   GPIO_InitStruct.GPIO_Speed = GPIO_High_Speed; 
   GPIO_InitStruct.GPIO_PuPd  = GPIO_PuPd_NOPULL;
   GPIO_InitStruct.GPIO_Pin   = RED_LED_PIN;
   
   GPIO_Init(RED_LED_PORT, &GPIO_InitStruct); 

   // ---
   // Amber LED

   // Enable peripheral clock 
   RCC_AHB1PeriphClockCmd(AMBER_LED_CLOCK, ENABLE);
     
   // Configure port pin for the LED
   GPIO_InitStruct.GPIO_Mode  = GPIO_Mode_OUT;
   GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
   GPIO_InitStruct.GPIO_Speed = GPIO_High_Speed; 
   GPIO_InitStruct.GPIO_PuPd  = GPIO_PuPd_NOPULL;
   GPIO_InitStruct.GPIO_Pin   = AMBER_LED_PIN;
   
   GPIO_Init(AMBER_LED_PORT, &GPIO_InitStruct); 

   // ---
   // Green LED

   // Enable peripheral clock 
   RCC_AHB1PeriphClockCmd(GREEN_LED_CLOCK, ENABLE);
     
   // Configure port pin for the LED
   GPIO_InitStruct.GPIO_Mode  = GPIO_Mode_OUT;
   GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
   GPIO_InitStruct.GPIO_Speed = GPIO_High_Speed; 
   GPIO_InitStruct.GPIO_PuPd  = GPIO_PuPd_NOPULL;
   GPIO_InitStruct.GPIO_Pin   = GREEN_LED_PIN;
   
   GPIO_Init(GREEN_LED_PORT, &GPIO_InitStruct); 

   // Configure initial state
   Tlight_sub_mode_g = START_STATE;  // Decide on initial state
   }
  

/*----------------------------------------------------------------------------*-

  TLIGHTS_Update()

  Controls state of traffic lights (represented by LEDs on board)

  Must schedule every second (soft deadline).
   
  PARAMETERS:
     None.

  LONG-TERM DATA:
     TLights_state_s (W)

  MCU HARDWARE:
     GPIO pin (Heartbeat pin).

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  WCET:
     Not yet determined.

  BCET:
     Not yet determined.

  RETURN VALUE:
     None.
 
-*----------------------------------------------------------------------------*/
void TLIGHTS_Update(void)
   {
   static uint32_t Time_in_sub_mode_s = 0;
     
   switch (Tlight_sub_mode_g)
      {
      case RED: 
         {
         GPIO_SetBits(RED_LED_PORT, RED_LED_PIN);
         GPIO_ResetBits(AMBER_LED_PORT, AMBER_LED_PIN);
         GPIO_ResetBits(GREEN_LED_PORT, GREEN_LED_PIN);

         // Check bulb outputs are correct
         TLIGHTS_Check_Bulbs();

         if (++Time_in_sub_mode_s == RED_DURATION)
            {
            Tlight_sub_mode_g = RED_AND_AMBER;
            Time_in_sub_mode_s = 0;
            }

         break;
         }

      case RED_AND_AMBER: 
         {
         GPIO_SetBits(RED_LED_PORT, RED_LED_PIN);
         GPIO_SetBits(AMBER_LED_PORT, AMBER_LED_PIN);
         GPIO_ResetBits(GREEN_LED_PORT, GREEN_LED_PIN);

         // Check bulb outputs are correct
         TLIGHTS_Check_Bulbs();

         if (++Time_in_sub_mode_s == RED_AND_AMBER_DURATION)
            {
            Tlight_sub_mode_g = GREEN;
            Time_in_sub_mode_s = 0;
            }

         break;
         }

      case GREEN: 
         {
         GPIO_ResetBits(RED_LED_PORT, RED_LED_PIN);
         GPIO_ResetBits(AMBER_LED_PORT, AMBER_LED_PIN);
         GPIO_SetBits(GREEN_LED_PORT, GREEN_LED_PIN);

         // Check bulb outputs are correct
         TLIGHTS_Check_Bulbs();

         if (++Time_in_sub_mode_s == GREEN_DURATION)
            {
            Tlight_sub_mode_g = AMBER;
            Time_in_sub_mode_s = 0;
            }

         break;
         }

      case AMBER: 
         {
         GPIO_ResetBits(RED_LED_PORT, RED_LED_PIN);
         GPIO_SetBits(AMBER_LED_PORT, AMBER_LED_PIN);
         GPIO_ResetBits(GREEN_LED_PORT, GREEN_LED_PIN);

         // Check bulb outputs are correct
         TLIGHTS_Check_Bulbs();

         if (++Time_in_sub_mode_s == AMBER_DURATION)
            {
            Tlight_sub_mode_g = RED;
            Time_in_sub_mode_s = 0;
            }

         break;
         }
      }   
      
   }

/*----------------------------------------------------------------------------*-

  TLIGHTS_Check_Bulbs()

  Simple demo - simulates checking of bulb state (with fault injection).

  PARAMETERS:
     None.

  LONG-TERM DATA:
     Call_count_s (W)

  MCU HARDWARE:
     None.

  PRE-CONDITION CHECKS:
     None.

  POST-CONDITION CHECKS:
     None.

  ERROR DETECTION / ERROR HANDLING:
     None.

  WCET:
     Not yet determined.

  BCET:
     Not yet determined.

  RETURN VALUE:
     None.
 
-*----------------------------------------------------------------------------*/
void TLIGHTS_Check_Bulbs(void)
   {
   static uint32_t Call_count_s = 0;
     
   // Here we would perform checks on the bulb health / outputs
   
   // In this demo, we simply simulate a fault after 35 calls 
   if (++Call_count_s == 35)  
      {
      // Trigger move to Fail-Safe State ...
      while(1);
      }
   }

/*----------------------------------------------------------------------------*-
  ------------------------------ END OF FILE ---------------------------------
-*----------------------------------------------------------------------------*/
